"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var styled_components_1 = require("styled-components");
var TweenLite = require("gsap/umd/TweenLite");
// @ts-ignore
var ScrolltoPlugin = require('gsap/umd/ScrollToPlugin');
var utils_1 = require("./utils");
var CarouselWrapperDiv = styled_components_1.default.div(templateObject_1 || (templateObject_1 = tslib_1.__makeTemplateObject(["\n  padding: 40vh 0;\n  margin: 0 auto;\n  width: 900px;\n  max-width: 100%;\n  min-height: 100vh;\n  opacity: ", ";\n  transition: opacity 300ms 100ms ease-in-out;\n  ", "\n"], ["\n  padding: 40vh 0;\n  margin: 0 auto;\n  width: 900px;\n  max-width: 100%;\n  min-height: 100vh;\n  opacity: ", ";\n  transition: opacity 300ms 100ms ease-in-out;\n  ", "\n"])), function (props) { return (props.isVisible ? 1 : 0); }, function (props) { return props.cssStyles; });
var ActiveChildWrapperClassname = 'Active';
var ChildWrapperDiv = styled_components_1.default.div(templateObject_2 || (templateObject_2 = tslib_1.__makeTemplateObject(["\n  position: relative;\n  width: 100%;\n  opacity: 0.4;\n  transition: opacity 50ms ease-in-out;\n\n  &.", " {\n    opacity: 1;\n  }\n  ", "\n"], ["\n  position: relative;\n  width: 100%;\n  opacity: 0.4;\n  transition: opacity 50ms ease-in-out;\n\n  &.", " {\n    opacity: 1;\n  }\n  ", "\n"])), ActiveChildWrapperClassname, function (props) { return props.cssStyles; });
exports.DefaultScrollOptions = {
    onScrollStart: function () { return null; },
    onScrollEnd: function () { return null; },
    forceScroll: false,
};
var Carousel = /** @class */ (function (_super) {
    tslib_1.__extends(Carousel, _super);
    function Carousel(props) {
        var _this = _super.call(this, props) || this;
        /**
         * getScrollYTarget determines the target scrollY for the element
         * at a specific index.
         *
         * If the element is smaller than the screen height, it will be centered on screen.
         * If it is not, the element will be at the top of the page.
         * @param index
         */
        _this.getScrollYTarget = function (index) {
            var wrapperElement = _this.wrapperRef.current;
            if (wrapperElement) {
                var element = wrapperElement.children.item(index);
                var targetWindowPosition = window.innerHeight *
                    (_this.props.activeChildBoundaryRatio || Carousel.defaultProps.activeChildBoundaryRatio);
                var currentElementHeight = element.getBoundingClientRect().height;
                var elementPivotPosition = currentElementHeight / 2;
                var isElementOversized = _this.props.isElementOversized ||
                    (function (elementHeight, clientHeight) { return elementHeight > clientHeight; });
                var getElementOffset = _this.props.getElementOffset || (function (isOversized) { return 0; });
                // If the element is oversized, return
                if (isElementOversized(currentElementHeight, window.innerHeight)) {
                    return element.offsetTop + getElementOffset(true);
                }
                return (Math.abs(element.offsetTop - targetWindowPosition + elementPivotPosition) +
                    getElementOffset(false));
            }
            return null;
        };
        /**
         * scrollToIndex uses a scroll tween to scroll to the element corresponding to
         * the index parameter
         * @param targetIndex
         * @param source
         * @param options,
         */
        _this.scrollToIndex = function (targetIndex, source, options) {
            var _a = Object.assign({}, exports.DefaultScrollOptions, options), forceScroll = _a.forceScroll, onScrollStart = _a.onScrollStart, onScrollEnd = _a.onScrollEnd;
            // Exit if already in a scroll animation
            if (!forceScroll && _this.state.isScrolling) {
                return;
            }
            var _b = _this.props, onScrollAnimationEnd = _b.onScrollAnimationEnd, onScrollAnimationStart = _b.onScrollAnimationStart, activeChildIndex = _b.activeChildIndex, setActiveIndex = _b.setActiveIndex, scrollAnimationDuration = _b.scrollAnimationDuration, scrollTweenOverwrite = _b.scrollTweenOverwrite, animatedUserScroll = _b.animatedUserScroll, disableAutoKill = _b.disableAutoKill, shouldScrollToIndex = _b.shouldScrollToIndex;
            try {
                var scrollContainer_1 = _this.getScrollContainerElement();
                var endingY_1 = _this.getScrollYTarget(targetIndex);
                if (scrollContainer_1 &&
                    endingY_1 &&
                    (!shouldScrollToIndex || shouldScrollToIndex(targetIndex, activeChildIndex, source))) {
                    var startingY_1 = scrollContainer_1.scrollTop;
                    _this.setState({
                        isScrolling: true,
                    }, function () {
                        // Run start callback
                        onScrollStart(activeChildIndex);
                        onScrollAnimationStart && onScrollAnimationStart(activeChildIndex);
                        var timer = setTimeout(function () {
                            _this.setState({ isScrolling: false });
                        }, scrollAnimationDuration ? scrollAnimationDuration + 0.1 : 0.4);
                        // Run tween animation explicitly from current child scroll position to the next
                        TweenLite.fromTo(scrollContainer_1, scrollAnimationDuration || 0.3, {
                            scrollTo: startingY_1,
                        }, {
                            scrollTo: {
                                y: endingY_1,
                                autoKill: !animatedUserScroll && !disableAutoKill,
                            },
                            overwrite: scrollTweenOverwrite || 'concurrent',
                            ease: 'Strong.easeInOut',
                            onComplete: function () {
                                _this.setState({
                                    isScrolling: false,
                                }, function () {
                                    clearTimeout(timer);
                                    setActiveIndex(targetIndex);
                                    onScrollEnd(targetIndex);
                                    onScrollAnimationEnd && onScrollAnimationEnd(targetIndex);
                                });
                            },
                        });
                    });
                }
            }
            catch (e) {
                console.warn(e);
            }
        };
        /**
         * getChildElements gets the child elements of the carousel wrapper
         * @returns {Array<Element> | null}
         */
        _this.getChildElements = function () {
            var wrapperElement = _this.wrapperRef.current;
            if (wrapperElement) {
                var elements = [];
                for (var i = 0; i < wrapperElement.children.length; i++) {
                    var item = wrapperElement.children.item(i);
                    if (item) {
                        elements.push(item);
                    }
                }
                return elements;
            }
            return null;
        };
        /**
         * getContainerElementOrFallback uses an id to get a specific DOM element
         * @param id: the id of the element
         * @param fallback: the fallback element
         */
        _this.getContainerElementOrFallback = function (id, fallback) {
            return document.getElementById(id) || fallback;
        };
        /**
         * getEventListenersTarget gets the DOM element for attaching the scroll and resize
         * event listeners
         */
        _this.getEventListenersTarget = function () {
            var scrollContainerId = _this.props.scrollContainerId;
            return scrollContainerId
                ? _this.getContainerElementOrFallback(scrollContainerId, window)
                : window;
        };
        /**
         * getScrollContainerElement gets the DOM element to use as the target
         * for the scroll tween
         */
        _this.getScrollContainerElement = function () {
            var scrollContainerId = _this.props.scrollContainerId;
            return scrollContainerId
                ? _this.getContainerElementOrFallback(scrollContainerId, document.documentElement)
                : document.documentElement;
        };
        _this.getMiddleElementStruct = utils_1.generateGetMiddleElementStruct(_this.getScrollContainerElement(), _this.props.activeChildBoundaryRatio || Carousel.defaultProps.activeChildBoundaryRatio);
        /**
         * opaqueChildIndexHandler
         * @param {Event} event
         */
        _this.opaqueChildIndexHandler = function (event) {
            var opaqueChildIndex = _this.state.opaqueChildIndex;
            var elements = _this.getChildElements();
            if (elements) {
                // Determine the middle element and its index
                var middleElementStruct = _this.getMiddleElementStruct(elements, event);
                if (middleElementStruct && middleElementStruct.index !== opaqueChildIndex) {
                    _this.setState({
                        opaqueChildIndex: middleElementStruct.index,
                    });
                }
            }
        };
        _this.opaqueChildScrollHandler = utils_1.debounce(_this.opaqueChildIndexHandler, 15);
        _this.opaqueChildResizeHandler = utils_1.debounce(_this.opaqueChildIndexHandler, 50);
        _this.wrapperRef = React.createRef();
        _this.state = {
            isClient: !!document,
            isScrolling: false,
            isVisible: false,
            opaqueChildIndex: 0,
        };
        return _this;
    }
    Carousel.prototype.componentDidMount = function () {
        var _this = this;
        this.setState({
            isClient: true,
        });
        this.timer = setTimeout(function () {
            _this.setState({
                isVisible: true,
            });
            var wrapperNode = _this.wrapperRef.current;
            _this.getEventListenersTarget().addEventListener('scroll', _this.opaqueChildScrollHandler, {
                passive: true,
            });
            _this.getEventListenersTarget().addEventListener('resize', _this.opaqueChildResizeHandler, {
                passive: true,
            });
            if (wrapperNode && React.Children.count(_this.props.children) > 0) {
                _this.scrollToIndex(0, 'onload');
            }
        }, 300);
    };
    Carousel.prototype.componentDidUpdate = function (prevProps, prevState, snapshot) {
        var _a = this.props, activeChildIndex = _a.activeChildIndex, animatedUserScroll = _a.animatedUserScroll;
        var _b = this.state, opaqueChildIndex = _b.opaqueChildIndex, isScrolling = _b.isScrolling;
        // If given a new activeChildIndex and it isn't the current opaque index,
        // scroll to it.
        if (prevProps.activeChildIndex !== activeChildIndex && activeChildIndex !== opaqueChildIndex) {
            this.scrollToIndex(activeChildIndex, 'props');
        }
        else if (prevState.opaqueChildIndex !== opaqueChildIndex &&
            animatedUserScroll &&
            !isScrolling) {
            this.scrollToIndex(opaqueChildIndex, 'scroll', {
                forceScroll: true,
            });
            // If the opaque child index changes (which only occurs via listeners)
            // Update the active index accordingly
        }
        else if (prevState.opaqueChildIndex !== opaqueChildIndex &&
            activeChildIndex !== opaqueChildIndex) {
            this.props.setActiveIndex(opaqueChildIndex);
        }
    };
    Carousel.prototype.componentWillUnmount = function () {
        window.removeEventListener('scroll', this.opaqueChildScrollHandler);
        window.removeEventListener('resize', this.opaqueChildResizeHandler);
        clearTimeout(this.timer);
    };
    Carousel.prototype.render = function () {
        var _this = this;
        var _a = this.props, children = _a.children, wrapperCss = _a.wrapperCss, wrapperProps = _a.wrapperProps, childWrapperCss = _a.childWrapperCss, _b = _a.childWrapperProps, childWrapperProps = _b === void 0 ? {} : _b, getChildZIndex = _a.getChildZIndex;
        var _c = this.state, isClient = _c.isClient, isVisible = _c.isVisible, opaqueChildIndex = _c.opaqueChildIndex;
        if (!isClient) {
            return null;
        }
        return (React.createElement(CarouselWrapperDiv, tslib_1.__assign({ isVisible: isVisible, cssStyles: wrapperCss, ref: this.wrapperRef }, wrapperProps), React.Children.map(children, function (child, index) {
            var mergedStyle = tslib_1.__assign({}, (childWrapperProps.style || {}));
            if (getChildZIndex) {
                mergedStyle.zIndex = getChildZIndex(index);
            }
            return (React.createElement(ChildWrapperDiv, tslib_1.__assign({ onClick: function (event) { return _this.scrollToIndex(index, 'click'); }, className: opaqueChildIndex === index ? ActiveChildWrapperClassname : '', cssStyles: childWrapperCss }, childWrapperProps, { style: mergedStyle }), child));
        })));
    };
    Carousel.defaultProps = {
        activeChildBoundaryRatio: 0.45,
    };
    return Carousel;
}(React.PureComponent));
exports.Carousel = Carousel;
exports.default = Carousel;
var templateObject_1, templateObject_2;
//# sourceMappingURL=index.js.map