"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * debounce is a generic implementation of a "debounce" pattern
 * to prevent calls of
 * @param func
 * @param wait
 * @param immediate
 */
exports.debounce = function (func, wait, immediate) {
    if (immediate === void 0) { immediate = false; }
    var timeout;
    return function () {
        // @ts-ignore
        var context = this, args = arguments;
        var later = function () {
            timeout = null;
            if (!immediate) {
                // @ts-ignore
                func.apply(context, args);
            }
        };
        var callNow = immediate && !timeout;
        clearTimeout(timeout);
        timeout = setTimeout(later, wait);
        if (callNow) {
            // @ts-ignore
            func.apply(context, args);
        }
    };
};
/**
 *
 * @param {HTMLElement} scrollContainerElement
 * @param {number} boundaryRatio
 * @returns {(elements: Array<Element>, event?: Event) => (ElementIndexStruct | null)}
 */
exports.generateGetMiddleElementStruct = function (scrollContainerElement, boundaryRatio) {
    // Define viewportHeight in function generator
    var viewportHeight = scrollContainerElement.clientHeight;
    return function (elements, event) {
        if (event && event.type == 'resize') {
            // Update viewportHeight defined in the closure of the outer function on resize only, not scroll
            viewportHeight = scrollContainerElement.clientHeight;
        }
        for (var i = 0; i < elements.length; i++) {
            var element = elements[i];
            var pos = element.getBoundingClientRect();
            var topBoundary = viewportHeight * boundaryRatio;
            var bottomBoundary = viewportHeight * boundaryRatio;
            var inViewport = pos.top <= topBoundary && pos.bottom >= bottomBoundary;
            if (inViewport) {
                return {
                    element: element,
                    index: i,
                };
            }
        }
        return null;
    };
};
//# sourceMappingURL=utils.js.map